@@ -28,7 +28,7 @@ DEFAULT_JWT_SESSION_HEADER_NAME = "Context"
 DEFAULT_JWT_SESSION_HEADER_TYPE = "Bearer"
 
 DEFAULT_JWT_API_KEY_ALGORITHM = "HS256" ###- HS512
-DEFAULT_JWT_API_KEY_HEADER_NAME = "Context"
+DEFAULT_JWT_API_KEY_HEADER_NAME = "Api-key"
 DEFAULT_JWT_API_KEY_HEADER_TYPE = "Bearer"
 
 DEFAULT_JWT_SECURITY_ALGORITHM = "HS256" ###- HS512
